package XML::Parser;

use strict 'vars', 'syms';
use vars qw($VERSION @ISA);
use Carp;

require DynaLoader;

@ISA = qw(DynaLoader);

$VERSION = "1.00" ;

bootstrap XML::Parser $VERSION;

sub AUTOLOAD {
    print "$XML::Parser::AUTOLOAD(@_)\n";
}

sub new {
    my ($class, %args) = @_;
    my $style = $args{Style} || "debug";
    $args{RawEvents} ||= "XML::Parser::\u$style";
    my $raw = $args{RawEvents};
    croak "undefined style: $style" unless defined &{$raw . "::char"};
    my $userdata;
    my $self = bless \%args, $_[0];
    my $rawent = $args{RawEntities} || 0;
    my $parser = ParserCreate("UTF-8", $self,
	*{$raw . "::start"},
	*{$raw . "::end"},
	*{$raw . "::char"},
	*{$raw . "::proc"},
	0, $rawent, $userdata);
    $args{Parser} = $parser;
    $args{Userdata} = $userdata;
    &{ $raw . "::init" }($self);
    $args{Pkg} ||= caller;
    $self;
}

sub DESTROY {
    my $self = shift;
    ParserFree($self->{Parser}, $self->{Userdata});
}

sub parsestring {
    my $self = shift;
    my $parser = $self->{Parser};
    unless (Parse($parser, $_[0], 1)) {
	my $err = ErrorString(GetErrorCode($parser));
	$err .= " at line " . GetErrorLineNumber($parser);
	$err .= " column " . GetErrorColumnNumber($parser);
	$err .= " byte " . GetErrorByteIndex($parser);
	&{ $self->{RawEvents} . "::final" }($self);
	die $err;
    }
    &{ $self->{RawEvents} . "::final" }($self);
    1;
}

sub parsefile {
    my $self = shift;
    my $parser = $self->{Parser};
    local(*FILE);
    open(FILE, $_[0]) or croak "$!";
    my $buf;
    while (sysread(FILE, $buf, 32768)) {
	unless (Parse($parser, $buf, length $buf < 32768)) {
	    my $err = ErrorString(GetErrorCode($parser));
	    $err .= " at line " . GetErrorLineNumber($parser);
	    $err .= " column " . GetErrorColumnNumber($parser);
	    $err .= " byte " . GetErrorByteIndex($parser);
	    &{ $self->{RawEvents} . "::final" }($self);
	    die $err;
	}
    }
    &{ $self->{RawEvents} . "::final" }($self);
    1;
}

###################################################################

package XML::Parser::Debug;

sub init {
    my $self = shift;
    $self->{Context} = [];
}

sub start {
    my $self = shift;
    my $tag = shift;
    push(@{$self->{Context}}, $tag);
    print STDERR "@{$self->{Context}} \\\\ (@_)\n";
}

sub end {
    my $self = shift;
    my $tag = shift;
    print STDERR "@{$self->{Context}} //\n";
    die "panic: context botch" unless pop @{$self->{Context}} eq $tag;
}

sub char {
    my $self = shift;
    my $text = shift;
    $text =~ s/([\x80-\xff])/sprintf "#x%X;", ord $1/eg;
    $text =~ s/([\t\n])/sprintf "#%d;", ord $1/eg;
    print STDERR "@{$self->{Context}} || $text\n";
}

sub proc {
    my $self = shift;
    my $target = shift;
    my $text = shift;
    print $self,"\n";
    print $self->{Context}, "\n";
    my @foo = @{$self->{Context}};
    print STDERR "@foo $target($text)\n";
}

sub final {
}

###################################################################

package XML::Parser::Subs;

sub init {
    my $self = shift;
    $self->{Context} = [];
}

sub start {
    my $self = shift;
    my $tag = shift;
    my $sub = $self->{Pkg} . "::$tag";
    eval { &$sub($self, $tag, @_) };
    push(@{$self->{Context}}, $tag);
}

sub end {
    my $self = shift;
    my $tag = shift;
    my $sub = $self->{Pkg} . "::${tag}_";
    die "panic: context botch" unless pop @{$self->{Context}} eq $tag;
    eval { &$sub($self, $tag) };
}

sub char {
    my $self = shift;
    my $text = shift;
    my $sub = $self->{Pkg} . "::characters";
    eval { &$sub($self, $text) };
}

sub proc {
    my $self = shift;
    my $target = shift;
    my $text = shift;
    my $sub = $self->{Pkg} . "::processing";
    eval { &$sub($self, $target, $text) };
}

sub final {
}

###################################################################

package XML::Parser::Tree;

sub init {
    my $self = shift;
    $self->{Lists} = [];
    $self->{Curlist} = $self->{Tree} = [];
}

sub start {
    my $self = shift;
    my $tag = shift;
    my $newlist = [ { @_ } ];
    push @{ $self->{Lists} }, $self->{Curlist};
    push @{ $self->{Curlist} }, $tag => $newlist;
    $self->{Curlist} = $newlist;
}

sub end {
    my $self = shift;
    my $tag = shift;
    $self->{Curlist} = pop @{ $self->{Lists} };
}

sub char {
    my $self = shift;
    my $text = shift;
    push @{ $self->{Curlist} }, 0 => $text;
}

sub proc {
    my $self = shift;
    my $target = shift;
    my $text = shift;
}

sub final {
    my $self = shift;
    delete $self->{Curlist};
    delete $self->{Lists};
}

###################################################################

package XML::Parser::Objects;

sub init {
    my $self = shift;
    $self->{Lists} = [];
    $self->{Curlist} = $self->{Tree} = [];
}

sub start {
    my $self = shift;
    my $tag = shift;
    my $newlist = [ ];
    my $newobj = bless { @_, Kids => $newlist }, $tag;
    push @{ $self->{Lists} }, $self->{Curlist};
    push @{ $self->{Curlist} }, $newobj;
    $self->{Curlist} = $newlist;
}

sub end {
    my $self = shift;
    my $tag = shift;
    $self->{Curlist} = pop @{ $self->{Lists} };
}

sub char {
    my $self = shift;
    my $text = shift;
    push @{ $self->{Curlist} }, bless { Text => $text }, 'characters';
}

sub proc {
    my $self = shift;
    my $target = shift;
    my $text = shift;
}

sub final {
    my $self = shift;
    delete $self->{Curlist};
    delete $self->{Lists};
}

1;

__END__

=head1 NAME

XML::Parser - Access to James Clark's xmltok/xmlparse XML parser

=head1 SYNOPSIS

 use XML::Parser;

 $parser = new XML::Parser;

 TBD

=head1 DESCRIPTION

=cut
